// Game.cpp: implementation of the cGame class and children.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "gamedambuilder.h"
#include "critterarmed.h"
#include "critterwall.h"
#include "force.h"
#include "listener.h"
#include "pop.h" //for playSound
#include "popview.h"
#include "spritebubble.h"
#include "critterviewer.h"
#include "graphicsopengl.h"
#include "graphicsmfc.h"
#include "spritequake.h"

IMPLEMENT_SERIAL( cGameDambuilder, cGame, 0);
IMPLEMENT_SERIAL( cCritterDamFloat, cCritter, 0);
IMPLEMENT_SERIAL( cCritterDamPlayer, cCritterArmedPlayer, 0);
#ifdef TESTBLOCK
IMPLEMENT_SERIAL( cCritterDamWall, cCritterBlock, 0);
#else
IMPLEMENT_SERIAL( cCritterDamWall, cCritterWall, 0);
#endif
IMPLEMENT_SERIAL( cCritterDamPlayerBullet, cCritterBullet, 0);

//#define DAMPLAYERGRAVITY
	/* If we define DAMPLAYERGRAVITYin, then the player will be subject to gravity.
	 You will only notice this with the ListenerCar or ListenerSpaceship control. 
	 Car won't look good, as it will be confusing when the car is falling backwards and
	it thinks it's in reverse. */

#define PROPGRAVITY
	/* If you want to turn off gravity for the props & bu, turn off PROPGRAVITY. */

//#define DEBUGDAM
	/* Turn on DEBUGDAM to turn off all but one of the dam walls and all the floats. */

//#define SPRITEQUAKE
	/* Use Quake sprites for the critters. Only do this if you plan to start in 3D. */

Real cGameDambuilder::WALLTHICKNESS = 0.4;
Real cGameDambuilder::PLAYERRADIUS = 0.5;
Real cGameDambuilder::MAXPLAYERSPEED = 30.0;
Real cGameDambuilder::WALLPRISMDZ = 1.0;
//==============================cGameDambuilder=======
cGameDambuilder::cGameDambuilder()
{
	_menuflags &= ~cGame::MENU_SHIELD;
		/* Default _menuflags from cGame has hopper off, autoplay off.
			We don't use the shield control in the standard game as nobody is
			shooting at us. */
#ifdef DAMPLAYERGRAVITY
	_menuflags |= cGame::MENU_HOPPER; 
		// The Hopper may be interesting if player has gravity.
#endif //DAMPLAYERGRAVITY
//Pick the cursor tools.
	//_arrayHCURSOR has only the _hCursorArrow inherited from cGame.
	_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorPin);
	_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorDragger);
	_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorReplicate);
//Initialize other fields
	setBorder( 25, 35) ; /* Use a flat world.  Note that
		setBorder sets the skybox to have a zrange (-0.1 to 1.0). */
	//I make the walls four shades .
	pskybox()->setSideSolidColor(LOX, RGB(180, 30, 200));
	pskybox()->setSideSolidColor(HIX, RGB(30, 10, 180));
	pskybox()->setSideSolidColor(LOY, RGB(255, 200, 200));
	pskybox()->setSideSolidColor(HIY, RGB(10, 255, 200));
	//Color the floor one of the following ways
	pskybox()->setSideTexture(LOZ, IDB_BIGWATER_1TO2);
//	pskybox()->setSideTexture(LOZ, IDB_BIGSKY);
//	pskybox()->setSideSolidColor(LOZ, RGB(128, 200, 256));
	//Make the top side transparent
	pskybox()->setSideInvisible(HIZ);

	setWrapflag(cCritter::WRAP);
	_spritetype = cGame::ST_SPHERES;
	_seedcount = cGame::COUNTLARGE;
#ifdef SPRITEQUAKE
	_spritetype = cGame::ST_MESHSKIN;
	_seedcount = cGame::COUNTSMALL; //Will run slow on some cards with many cSpriteQuake
#endif //SPRITEQUAKE
//Set the player 
	setPlayer(new cCritterDamPlayer());
//Put in some walls in the constructor.
#ifndef DEBUGDAM
	new cCritterDamWall( cVector(-4, 2),	cVector(3, -2), cGameDambuilder::WALLTHICKNESS, this);
	new cCritterDamWall( cVector(1.5, -2),	cVector(6, 0.3), cGameDambuilder::WALLTHICKNESS, this);
	new cCritterDamWall( cVector(-5, 0),	cVector(-5, 9), cGameDambuilder::WALLTHICKNESS, this);
	new cCritterDamWall( cVector(8, -2),	cVector(12, -3), cGameDambuilder::WALLTHICKNESS, this);
	new cCritterDamWall( cVector(-8, -4),	cVector(-6, -3), cGameDambuilder::WALLTHICKNESS, this);
	new cCritterDamWall( cVector(-9, -8),	cVector(-3, -3), cGameDambuilder::WALLTHICKNESS, this);
#else //DEBUGDAM is true  Just have one wall.
	new cCritterDamWall( cVector(-4, 2),	cVector(4, 2), cGameDambuilder::WALLTHICKNESS, this);
	_seedcount = 0;
#endif //DEBUGDAM
}

void cGameDambuilder::seedCritters()
{
 	_pbiota->purgeNonPlayerNonWallCritters(); /* Clean out all critters but leave 
		player and	walls. */
	for (int i=0; i < _seedcount; i++)
		new cCritterDamFloat(this);
}

void cGameDambuilder::initializeView(CPopView *pview)
{
	cGame::initializeView(pview); //You MUST call the base class method here.
	pview->setCursor(((CPopApp*)::AfxGetApp())->_hCursorDragger);
	pview->setUseBackground(CPopView::FULL_BACKGROUND);/* The background type can be
		CPopView::NO_BACKGROUND, CPopView::SIMPLIFIED_BACKGROUND, or 
		CPopView::FULL_BACKGROUND, which often means: nothing, lines, or
		planes&bitmaps, depending on how the skybox is defined. */
	CPoint panepos = pview->paneColRow(); /* If you use Window|Split you'll
		get two panes, in columns 0 and 1, and both in row zero,
		thus their colrow positiosn are (0,0) and (1,0). */
	if (panepos == CPoint(0,0)) //Use flat map for first view.
	{
	//	pview->setGraphicsClass(RUNTIME_CLASS(cGraphicsOpenGL)); 
		//	pview->setUseBackground(CPopView::FULL_BACKGROUND);
			pview->setGraphicsClass(RUNTIME_CLASS(cGraphicsMFC)); 
	}
	if (panepos == CPoint(1,0)) 
	{
		pview->setGraphicsClass(RUNTIME_CLASS(cGraphicsOpenGL)); 
		//	pview->setUseBackground(CPopView::SIMPLIFIED_BACKGROUND);
		pview->pviewpointcritter()->setListener(new cListenerViewerRide());
	}
}

void cGameDambuilder::initializeViewpoint(cCritterViewer *pviewer)
{
	cGame::initializeViewpoint(pviewer); /* No overload.  Note that you don't actually
		care about this call if you plan to ride the player, as then the viewpoint is
		set by the player. */
	pviewer->zoom(1.0);
}

void cGameDambuilder::drawBackground(cGraphics *pgraphics, int drawflags)
{
	//pgraphics->pushMatrix();
///	pgraphics->multMatrix(cMatrix::translation(cVector(0.0, 0.0,
//		-cGameDambuilder::WALLPRISMDZ))); 
		/* Slide the floor down to account for the fact
		that the walls will be centered on the z=0 plane, half above, half below. */
	cGame::drawBackground(pgraphics, drawflags);
//	pgraphics->popMatrix();
}

//=====================cCritterDamFloat===========

cCritterDamFloat::cCritterDamFloat(cGame *pownergame):
cCritter(pownergame)
{
	if (pgame())
		setSprite(pgame()->randomSprite(pgame()->spritetype()));
  	psprite()->setLineColor(cColorStyle::CN_WHITE); /* White edges as we use
		a dark background.	Note that this color affect bullets you shoot, as
		well as your own sprite. */
	randomize(cCritter::MF_POSITION | cCritter::MF_VELOCITY); 
	addForce(new cForceEvadeBullet());
#ifdef PROPGRAVITY
	addForce(new cForceGravity()); //Uses default gravity strength 25.
	addForce(new cForceDrag()); //Uses default friction strength 0.5.
#endif //PROPGRAVITY
	setRadius(0.6);
	setDensity(2.0); 
	setMaxspeed(20.0);
}

//===================cCritterDamPlayer================
cCritterDamPlayer::cCritterDamPlayer(cGame *pownergame):
cCritterArmedPlayer(pownergame)
{
	setListener(new cListenerScooter());
  	psprite()->setLineColor(cColorStyle::CN_WHITE); /* White edges as we use
		a dark background.	Note that this color affect bullets you shoot, as
		well as your own sprite. */
 	setBulletClass(RUNTIME_CLASS(cCritterDamPlayerBullet));
	setWaitShoot(2.0*cCritterArmed::WAITSHOOT); //Don't shoot as fast as usual.
	setMaxspeed(cGameDambuilder::MAXPLAYERSPEED);
	setListenerAcceleration(160.0); //So Hopper can overcome gravity
	setRadius(cGameDambuilder::PLAYERRADIUS);
	setAbsorberflag(TRUE); //Keeps player from being buffeted about.
#ifdef DAMPLAYERGRAVITY
	setListenerAcceleration(80.0); //So Car or Spaceship can overcome gravity
	addForce(new cForceGravity()); //Uses default gravity strength 25.
	addForce(new cForceDrag()); //Uses default friction strength 0.5.
#endif //DAMPLAYERGRAVITY
}

//===================cCritterDamWall
cCritterDamWall::cCritterDamWall(const cVector &enda, const cVector &endb,
	 Real thickness, cGame *pownergame):
cCritterWall(enda, endb, thickness, cGameDambuilder::WALLPRISMDZ, pownergame)
{
	psprite()->setFillColor(cColorStyle::CN_ORANGE);
	/* The wall will be centered in the xy plane.  To make it sit up on top
	of the xy plane I have two options.  Either I can slide its sprite up
	with this line: */
//		psprite()->setSpriteAttitude(
//			cMatrix::translation(cVector(0.0, 0.0, height()/2.0)));
/*	or I can move the whole wall up with these 3 lines as in gameracer.cpp,
	(only these don't work well with replication):
	_movebox.setZRange(-100.0, 100.0);
		 // So you can move up.  100 is overkill.
	_dragbox.setZRange(-100.0, 100.0);
		 // So you can drag and not clamp.
	moveTo(cVector(_position.x(), _position.y(), -0.1 + height()/2.0));
		// Align base to be just a hair under the xy plane
*/
}

//====================cCritterDamPlayerBullet==================
cCritterDamPlayerBullet::cCritterDamPlayerBullet()
{
	setRadius(2.0*cCritter::BULLETRADIUS); //Make bigger than usual.
#ifdef PROPGRAVITY
	addForce(new cForceGravity()); //Uses default gravity strength 25.
	addForce(new cForceDrag()); //Uses default friction strength 0.5.
#endif //PROPGRAVITY
}

cCritterDamPlayerBullet::isTarget(cCritter *pcritter)
{
	return cCritterBullet::isTarget(pcritter) && 
		pcritter->GetRuntimeClass() != RUNTIME_CLASS(cCritterDamWall);
}
