// Gamehallway.cpp: implementation of the GameHallway class and associated classes.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "gamehallway.h"
#include "force.h"
#include "listener.h"
#include "pop.h" //for playSound
#include "spritepolygon.h" 
#include "critterviewer.h"
#include "graphicsopengl.h"
#include "graphicsmfc.h"
#include "spritequake.h"
#include "spritemultiicon.h"
#include "critterwall.h"
#include "sprite3d.h"

IMPLEMENT_SERIAL( cGameHallway, cGame, 0);
IMPLEMENT_SERIAL( cCritterHallwayProp, cCritter, 0);
IMPLEMENT_SERIAL( cCritterHallwayPlayer, cCritterArmedPlayer, 0);
IMPLEMENT_SERIAL( cCritterHallwayTreasure, cCritter, 0);
IMPLEMENT_SERIAL( cCritterHallwayPlayerBullet, cCritterBullet, 0);

//==============================cGameHallway=======
Real cGameHallway::TREASURERADIUS = 1.2;
Real cGameHallway::WALLTHICKNESS = 0.5;
Real cGameHallway::PLAYERRADIUS = 0.2;
Real cGameHallway::MAXPLAYERSPEED = 30.0;
/* 
If radius is 0.2, walls are 0.2 thick and maxspeed is 15.0, I can run through walls.  
If radius is 0.4, walls are 0.4 thick and maxspeed is 15.0, I can't run through walls.
If radius is 0.4, walls are 0.4 thick and maxspeed is 30.0, I can't run through walls.
If radius is 0.2, walls are 0.4 thick and maxspeed is 30.0, I can run through walls (hard). 
If radius is 0.2, walls are 0.2 thick and maxspeed is 30.0, I can run hrough walls (easy). 
 */ 
cGameHallway::cGameHallway()
{
//Fix the menu selections you'll allow.
//	_menuflags |= cGame::MENU_SHAPE; //Allow to change spriteshapes.
	_menuflags &= ~cGame::MENU_BOUNCEWRAP; 
		/* Default _menuflags from cGame has autoplay off, hopper off. 
		Turn off wrap because it's bad with gravity, we want balls bouncing on bottom.*/
	_menuflags &= ~cGame::MENU_ALLOWLISTENERCURSOR;
		/* 	We don't allow the listenercursor as it doesn't presently work
			right when you ride the player.  */
	_menuflags |= cGame::MENU_HOPPER; //Turn on hopper listener option.
	_spritetype = cGame::ST_MESHSKIN;
	setBorder(12.0, 12.0, 48.0); //Use a long, thin world. //was 12, 12, 100

	cRealBox3 skeleton = _border;
	setSkyBox(&skeleton);
/* In this world the coordinates are screwed up to match the screwed up
	listener that I use.  I should fix the listener and the coords.
	Meanwhile...
	I am flying into the screen from HIZ towards LOZ, and
	LOX below and HIX above and
	LOY on the right and HIY on the left. */
	pskybox()->setSideInvisible(HIZ); //Make the near HIZ transparent
	pskybox()->setSideTexture(LOZ, IDB_BEIGEFOAM); //Far wall
	pskybox()->setSideSolidColor(LOX, RGB(250, 200, 50)); //left wall
	pskybox()->setSideSolidColor(HIX, RGB(240, 255, 255)); //right wall
	pskybox()->setSideSolidColor(LOY, RGB(250, 240, 240)); //floor
	pskybox()->setSideTexture(HIY, IDB_AQUATRAPEZOIDS, 3); //ceiling
	//pskybox()->setSideSolidColor(HIY, RGB(240, 255, 255)); //ceiling

	setWrapflag(cCritter::BOUNCE);
	_seedcount = 7; 
	setPlayer(new cCritterHallwayPlayer(this));
	_ptreasure = new cCritterHallwayTreasure(this);
	//_arrayHCURSOR has only the _hCursorArrow inherited from cGame.
	//_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorPlay); //No need for the shoot cursor

	/* In this world the x and y go left and up respectively, while z comes out of the screen.
	A wall views its "thickness" as in the y direction, which is up here, and its
	"height" as in the z direction, which is into the screen. */
	//First draw a net with dy height resting on the bottom of the world.
	Real netzpos = 0.0; /* Point on the z axis where we set down the wall.  0 would be center,
		halfway down the hall, but we can offset it if we like. */
	Real netheight = 0.1*_border.ysize();
	Real ycenter = -_border.yradius() + netheight/2.0;
	Real wallthickness = cGameHallway::WALLTHICKNESS;
	cCritterWall *pwall = new cCritterWall( 
		cVector(_border.lox() + 2.0, ycenter , netzpos + 4.0),
		cVector(_border.hix(), ycenter, netzpos),
		netheight, //thickness param for wall's dy which goes perpendicular to the
			//baseline established by the frist two args, up the screen
		wallthickness,  //height argument for this wall's dz  goes into the screen
		this);
	pwall->setFillColor(cColorStyle::CN_LIGHTRED);
	cSpriteTextureBox *pspritebox =
		new cSpriteTextureBox(pwall->pskeleton(), IDB_SMOOTHBRICK, 6); //Sets all sides
		/* We'll tile our sprites three times along the long sides, and on the
		short ends, we'll only tile them once, so we reset these two. */
//	pspritebox->setSideSolidColor(LOX, cColorStyle::CN_LIGHTRED);
//	pspritebox->setSideSolidColor(HIX, cColorStyle::CN_LIGHTRED);
	pwall->setSprite(pspritebox);


	//Then draw a ramp to the top of the net.  Scoot it over against the right wall.
#define PLANK
#ifdef PLANK
	Real planckwidth = 0.75 * netheight;
	pwall = new cCritterWall( 
		cVector(_border.hix()-planckwidth/2.0, _border.loy(), _border.hiz()),
		cVector(_border.hix()-planckwidth/2.0, _border.loy()+netheight, netzpos),
		planckwidth,  //thickness param for wall's dy which is perpenedicualr to the baseline,
				//which goes into the screen, so thickness goes to the right
		wallthickness, //_border.zradius(),  //height argument for wall's dz which goes into the screen
		this);
	pwall->setFillColor(cColorStyle::CN_LIGHTGREEN);
#endif //PLANK
//	pwall->rotate(cSpin(PI/4.0, cVector::YAXIS));
	
//	new cCritterWall( cVector(1.5, -2, 7),	cVector(6, 0.3, 7), 0.5, 2.0, this);
//	new cCritterWall( cVector(-5, 0),	cVector(-5, 9), 0.5, 1.5, this);
//	new cCritterWall( cVector(8, -2, -4),	cVector(12, -3, -10), 0.5, 2.5, this);
//	new cCritterWall( cVector(-8, -4, -20),	cVector(-6, -3, -20), 0.5, 1.0, this);
//	new cCritterWall( cVector(-9, -8, -30),	cVector(-3, -3, -30), 0.5, 3.0, this);

}

void cGameHallway::initializeView(CPopView *pview)
{
	cGame::initializeView(pview); //You MUST call the base class method here.
	pview->setUseBackground(CPopView::FULL_BACKGROUND);/* The background type can be
		CPopView::NO_BACKGROUND, CPopView::SIMPLIFIED_BACKGROUND, or 
		CPopView::FULL_BACKGROUND, which often means: nothing, lines, or
		planes&bitmaps, depending on how the skybox is defined. */
	CPoint panepos = pview->paneColRow();
	if (panepos == CPoint(0,0))
	{
		pview->setGraphicsClass(RUNTIME_CLASS(cGraphicsOpenGL));
		pview->pviewpointcritter()->setListener(new cListenerViewerRide());
	}
	if (panepos == CPoint(1,0)) //Use flat map for second view.
	{
		pview->setGraphicsClass(RUNTIME_CLASS(cGraphicsOpenGL)); 
		pview->setUseBackground(CPopView::SIMPLIFIED_BACKGROUND);/* The background type can be
		CPopView::NO_BACKGROUND, CPopView::SIMPLIFIED_BACKGROUND, or 
		CPopView::FULL_BACKGROUND, which often means: nothing, lines, or
		planes&bitmaps, depending on how the skybox is defined. */
		pview->pviewpointcritter()->setTrackplayer(TRUE); 
		/* Always look at the player. 
			Don't comment this in unless you plan to be zoomed in. */
	}
}

void cGameHallway::initializeViewpoint(cCritterViewer *pviewer)
{
	if (pviewer->plistener()->GetRuntimeClass()
		== (RUNTIME_CLASS(cListenerViewerRide)))
	{
		pviewer->setViewpoint(cVector3(0.0, 0.3, -1.0), _border.center());
			//Always make some setViewpoint call simply to put in a default zoom.
		pviewer->zoom(0.35); //Wideangle
		cListenerViewerRide *prider = (cListenerViewerRide*)(pviewer->plistener());
		prider->setOffset((-1.5* cVector::XAXIS + 1.0 * cVector::ZAXIS)); /* This offset is in the coordinate
			system of the player, where the negative X axis is the negative of the
			player's tangent direction, which means stand right behind the player. */
	}
	else //Not riding the player.
	{
		pviewer->zoom(1.5);
		pviewer->setViewpoint(cVector::YAXIS, pplayer()->position());
	}
}

void cGameHallway::seedCritters()
{
	pbiota()->purgeCritters(RUNTIME_CLASS(cCritterBullet));
	pbiota()->purgeCritters(RUNTIME_CLASS(cCritterHallwayProp));
	for (int i=0; i<_seedcount; i++)
		new cCritterHallwayProp(this);
	pplayer()->moveTo(cVector(0.0, border().loy(), border().hiz() - 3.0)); 
		/* We start at hiz and move towards	loz */
}

void cGameHallway::adjustGameParameters()
{
// (1) End the game if the player is dead
	if (!health()  && !_gameover) //Player's been killed and game's not over.
	{
		_gameover = TRUE;
		pplayer()->addScore(_scorecorrection); // So user can reach _maxscore 
		playSound("Tada");
		return; 
	}
// (2) Also don't let the ballcount diminish.
			//(need to recheck propcount in case we just called seedCritters).
	int ballcount = pbiota()->count(RUNTIME_CLASS(cCritterHallwayProp));
	int ballstoadd = _seedcount - ballcount;
 	for (int i=0; i < ballstoadd; i++)
		new cCritterHallwayProp(this);
// (3) Maybe check some other conditions.
 }

CString cGameHallway::statusMessage()
{
	CString cStrStatusBar;
	int nUpdatesPerSecond;
	CString cStrUpdatespersecond;
	CString cStrCount;
	CString cStrScore;
	CString cStrHealth;

	if (!gamepaused())
	{
		nUpdatesPerSecond = int(((CPopApp*)::AfxGetApp())->_timer.updatesPerSecond());
		if (!nUpdatesPerSecond)
			cStrUpdatespersecond.Format("Less than one update per second.");
		else	
			cStrUpdatespersecond.Format("Updates per second: %d.", nUpdatesPerSecond);
		if (((CPopApp*)::AfxGetApp())->_timer.runningNearMinSpeed())
			cStrUpdatespersecond += " (Slower than Real Time)";
	}
	else
			cStrUpdatespersecond.Format("Animation is paused.");
	cStrScore.Format("Score: %d.", score());
	cStrHealth.Format("Health: %d.", health());
	int crittercount = _pbiota->count(RUNTIME_CLASS(cCritter));
	int bulletcount = _pbiota->count(RUNTIME_CLASS(cCritterBullet));
	cStrCount.Format("Ball Critters: %d.", crittercount-bulletcount-2); //Subtract two for player and treasure
	cStrStatusBar = cStrScore + cStrHealth + "  " + cStrCount + "  " + cStrUpdatespersecond; 
	return cStrStatusBar;
}

void cGameHallway::Serialize(CArchive& ar) /* This is a bit of work, as I have to match
	_ppuck , _phockeyrobot , _pmygoal , _probotgoal to the pointers in the cBiota array. */
{
	int treasureindex;
	cGame::Serialize(ar);
	if (ar.IsStoring()) // Save 
 	{
		ar << _index(_ptreasure);
	}
	else //Load
	{
		ar >> treasureindex;
			// I have to cast from cCritter*
		_ptreasure = (cCritterHallwayTreasure*)(_pbiota->GetAt(treasureindex));
	}
}
//====================Critters for the Hallway Game ==========================
//==================================================================


//==================cCritterHallwayPlayer=============

cCritterHallwayPlayer::cCritterHallwayPlayer(cGame *pownergame):
cCritterArmedPlayer(pownergame)
{
	_pbulletclass = RUNTIME_CLASS(cCritterHallwayPlayerBullet);
	setSprite(new cSpriteSphere());
	psprite()->setFillColor(cColorStyle::CN_DARKGRAY);
	psprite()->setSpriteAttitude(cMatrix::scale(2, 0.8, 0.4));
//	setSprite(new cSpriteQuake()); //Use default model whatever that is.
//	setSprite(new cSpriteQuakeOgro()); //Use Ogro
	setRadius(cGameHallway::PLAYERRADIUS); //Default cCritter::PLAYERRADIUS is 0.4. 
	setHealth(10);
	moveTo(_movebox.locorner() + 2.0* cVector::ZAXIS);
	setWrapflag(cCritter::CLAMP); //Use CLAMP so you stop dead at edges.
	setArmed(TRUE);  //Let's use bullets.
	setMaxspeed(cGameHallway::MAXPLAYERSPEED); 
	setAbsorberflag(TRUE); //Keeps player from being buffeted about.
	setListenerAcceleration(160.0); //So Hopper can overcome gravity.  Only affects hop.

//	setMaxspeed(25.0); //Need  speed to overcome gravity.  But not too fast or you punch thru wall
//	setListener(new cListenerHopper(20.0, 0.2, 0.2)); 
	setListener(new cListenerScooterYHopper(0.2, 0.2)); 
		/* The three arguments to the
		cListenerHopper constructor specify the walkspeed, the hopuptime, and the hopagainwait.
		Walkspeed is how fast the left/right (and PgUp/PgDn if in 3D where the ground is xz plane)
			arrow keys move the critter,
		Hopuptime is how many seconds you can accelerate up with the UP arrow, and
		Hopupwait is how many seconds you have to wait after the end of one hop
			before starting a new hop. 

		The cListenerScooterYHopper jsut takes hopuptime and hopagainwait. */
	 //setListener(new cListenerScooter());
		/* Or just use the good old scooter. */
	addForce(new cForceGravity(50.0)); /* Uses  gravity. Default strength is 25.0.
		Gravity	will affect player using cListenerHopper. */
//	addForce(new cForceDrag(0.5)); //Better without friction.
//	setDensity(8.0 * density()); //To make it fall faster, but we don't need this.
	setAttitudeToMotionLock(FALSE);  //It looks nicer is you don't turn the player with motion.
//	setAttitudeTangent(cVector::ZAXIS);
	 /* Without motion lock, we need setAttitudeTangent 
		to make sure he's pointing towards the goal at the start. We think of
		the hallway running from low Z to high Z, with the X and Y axes in
		the usual orientation.  In this case, a critter heading down the hall
		towards hi Z will have a k, i, j trihedron, which works as -k * (-i) =
		 j. */
	setAttitude(cMatrix(-cVector::ZAXIS, -cVector::XAXIS, cVector::YAXIS, position()));
}

BOOL cCritterHallwayPlayer::collide(cCritter *pcritter)
{
	BOOL playerhigherthancritter = position().y() - radius() > pcritter->position().y();
/* If you are "higher" than the pcritter, as in jumping on it, you get a poitn
and the critter dies.  If you are lower than it, you lose health and the
critter also dies. To be higher, let's say your low point has to higher
than the critter's center. We compute playerhigherthancritter before the collide,
as collide can change the positions. */
	BOOL collided = cCritter::collide(pcritter);
	if (!collided)
		return FALSE;
/* If you're here, you collided.  We'll treat all the guys the same -- the collision
 with a HallwayTreasure is different, but we let the HallwayTreasure contol that collision. */
//#define DEBUGTOUCH
#ifdef DEBUGTOUCH
	Real cz = pcritter->position().z();
	Real cr = pcritter->radius();
	cz += cr;
	Real mz = position().z();
	Real mr = pcritter->radius();
	mz -= mr;
#endif //DEBUGTOUCH
	if (playerhigherthancritter)
	{
		playSound("Ding");
		addScore(10);
	}
	else
	{
		damage(1);
		playSound("Bonk");
	}
	pcritter->die();
	return TRUE;
}

//===============bullet
void cCritterHallwayPlayerBullet::initialize(cCritterArmed *pshooter)
{
	cCritterBullet::initialize(pshooter);
	clearForcearray();
}

//==================cCritterHallwayProp=============
cCritterHallwayProp::cCritterHallwayProp(cGame *pownergame):
cCritter(pownergame)
{
	addForce(new cForceGravity(25.0, cVector( 0.0, -1, 0.03))); 
	/* Make the	gravity tilt towards positive z  to move them along. */
	addForce(new cForceDrag(0.3)); //Uses default friction strength 0.5.
	setDensity(2.0); 
	setMaxspeed(30.0);
   	randomize(cCritter::MF_VELOCITY);
	if (pownergame) //Just to be safe.
		setSprite(pgame()->randomSprite(pownergame->spritetype()));
		/* Let the game	set the current default sprite. */
	if (psprite()->IsKindOf(RUNTIME_CLASS(cSpriteQuake))) //Don't let the figurines tumble. 
	{
		setAttitudeToMotionLock(FALSE);  //Use this to always face HIZ 
		setAttitude(cMatrix(cVector::ZAXIS, cVector::XAXIS, cVector::YAXIS, position()));
		/* Orient them so they are running towards positive Z with heads towards Y. */
	}
	setBounciness(0.999); //Not 1.0 means it loses a bit of energy with each bounce.
	setRadius(0.3);
	//randomize(cSprite::MF_RADIUS);
	setMinTwitchThresholdSpeed(4.0); //Means sprite doesn't switch direction unless it's moving fast
	Real playerz = 0.0;
	if (pownergame)
		playerz = pownergame->pplayer()->position().z();
	randomizePosition(cRealBox(cVector(_movebox.lox(),_movebox.loy(), playerz + 3.0),
		cVector(_movebox.hix(),_movebox.hiy(), _movebox.loz()))); 
		/* I put them ahead of the player  */
	if (pownergame) //Then we know we added this to a game so pplayer() is valid
 		addForce(new cForceObjectSeek(pplayer(), 0.5));
}

void cCritterHallwayProp::update(CPopView *pactiveview, Real dt)
{
	cCritter::update(pactiveview, dt); //Always call this first
	if (_outcode & BOX_HIZ) /* use bitwise AND to check if a flag is set. */
		delete_me(); //tell the game to remove yourself if you fall up to the hiz.
}


void cCritterHallwayProp::die()
{
	pplayer()->addScore(value());
	cCritter::die();
}

//==================cCritterHallwayTreasure=============

cCritterHallwayTreasure::cCritterHallwayTreasure(cGame *pownergame):
cCritter(pownergame)
{
//#define USELOOPSPRITE
	/* The sprites look nice from afar, but bitmap speed is really slow
	when you get close to then, so don't use this. */
#ifndef USELOOPSPRITE
	cPolygon *ppoly = new cPolygon(24);
	ppoly->setFilled(FALSE);
	ppoly->setLineWidthWeight(0.1);
	setSprite(ppoly);
#else //USELOOPSPRITE
	cSpriteLoop *ploop = new cSpriteLoop();
	ploop->add(IDB_EXPLODE1);
	ploop->add(IDB_EXPLODE2);
	ploop->add(IDB_EXPLODE3);
	ploop->setFlipwait(0.05); //How fast to flip the frames
	setSprite(ploop);
#endif  //USELOOPSPRITE
	_collidepriority = cCollider::CP_PLAYER + 1; /* Let this guy call collide on the
		player, as his method is overloaded in a special way. */
	rotate(cSpin(PI/2.0, cVector::ZAXIS)); /* Trial and error shows this
		rotation works to make it face the z diretion. */
	setRadius(cGameHallway::TREASURERADIUS);
	setFixedflag(TRUE);
	moveTo(cVector(_movebox.midx(), _movebox.midy(),
		_movebox.loz()- 1.5 * cGameHallway::TREASURERADIUS));
}

int cCritterHallwayTreasure::collidesWith(cCritter *pothercritter)
{
	if (pothercritter->IsKindOf(RUNTIME_CLASS(cCritterHallwayPlayer)))
		return cCollider::COLLIDEASCALLER;
	else
		return cCollider::DONTCOLLIDE;
}

BOOL cCritterHallwayTreasure::collide(cCritter *pcritter)
{
	if (contains(pcritter)) //disk of pcritter is wholly inside my disk
	{
		playSound("Tada");
		pcritter->addScore(100);
		pcritter->addHealth(1);
		pcritter->moveTo(pgame()->border().locorner());
		return TRUE;
	}
	else
		return FALSE;
}
