// Game.cpp: implementation of the cGame class and children.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "gameracer.h"
#include "critterarmed.h"
#include "critterwall.h"
#include "force.h"
#include "listener.h"
#include "pop.h" //for playSound
#include "popview.h"
#include "spritebubble.h"
#include "critterviewer.h"
#include "graphicsopengl.h"
#include "graphicsmfc.h"
#include "spritequake.h"
#include "spritetexturebox.h"
#include "sprite3d.h"

IMPLEMENT_SERIAL( cGameRacer, cGame, 0);
IMPLEMENT_SERIAL( cCritterRacerCar, cCritterArmedRobot, 0);
IMPLEMENT_SERIAL( cCritterRacerPlayer, cCritterArmedPlayer, 0);
IMPLEMENT_SERIAL( cCritterRacerWall, cCritterWall, 0);
IMPLEMENT_SERIAL( cCritterRacerPlayerBullet, cCritterBullet, 0);
IMPLEMENT_SERIAL( cCritterCarBullet, cCritterBullet, 0);

//#define DEBUGRACE
	/* Turn on DEBUGRACE to turn off all but one of the dam walls and all the floats. */

//#define STARTIN2D
	/* Turn on STARTIN2D to start in 2D, otherwise start in 3D. */
//#define SPRITEQUAKE
	/* Use Quake sprites for the critters. Only do this if you plan to start in 3D. */
#define WEAKBOUNCE
	/* By default I set the player's absorberflag to TRUE so he doesn't bounce
	at all.  But I can bounce just a little by setting absorberflag to FALSE
	and _bounciness to, say, 0.2 */
#define CARGRAVITY

Real cGameRacer::WALLTHICKNESS = 0.4;
Real cGameRacer::PLAYERRADIUS = 0.5;
Real cGameRacer::MAXPLAYERSPEED = 30.0;
Real cGameRacer::WALLPRISMDZ = 1.0; //Use this as the average height of walls.
Real cGameRacer::RACERPLAYERBOUNCINESS = 0.05; /* Let's have the player
	bounce only a small amount off the walls and other players. */
//==============================cGameRacer=======
cGameRacer::cGameRacer()
{
	_menuflags &= ~(cGame::MENU_ALLOWLISTENERCURSOR);
		/* Default _menuflags from cGame has hopper off, autoplay off.
			We don't allow the listenercursor as it doesn't presently work
			right when you ride the player.  */
#ifdef CARGRAVITY
//	_menuflags |= cGame::MENU_HOPPER; 
		//The Hopper may be interesting if player has gravity.
		//But as of 35.1, hopper doesn't work in here, so don't allow it.
#endif //CARGRAVITY
//Pick the cursor tools.
	//_arrayHCURSOR has only the _hCursorArrow inherited from cGame.
	_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorPin);
	_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorDragger);
	_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorReplicate);
//Initialize other fields
	setBorder( cVector(-20, -20, 0), cVector(20, 20, 8)) ; 
	setWrapflag(cCritter::BOUNCE);
	_spritetype = cGame::ST_MESHSKIN;  //cGame::ST_SIMPLEPOLYGONS;
	_seedcount = cGame::COUNTLARGE;
#ifdef SPRITEQUAKE
	_spritetype = cGame::ST_MESHSKIN;
	_seedcount = cGame::COUNTSMALL; //Will run slow on some cards with many cSpriteQuake
#endif //SPRITEQUAKE
//Set the player 
	setPlayer(new cCritterRacerPlayer());
	pplayer()->moveTo(cVector(_border.locorner() + cVector(1,1,0)));
	pplayer()->lookAt(cVector::ZEROVECTOR);
//Put in some walls in the constructor.
#ifndef DEBUGRACE
	new cCritterRacerWall( cVector(-4, 2),	cVector(3, -2), this);
	new cCritterRacerWall( cVector(1.5, -2),	cVector(6, 0.3),this);
	new cCritterRacerWall( cVector(-5, 0),	cVector(-5, 9), this);
	new cCritterRacerWall( cVector(8, -2),	cVector(12, -3), this);
	new cCritterRacerWall( cVector(-8, -4),	cVector(-6, -3), this);
	new cCritterRacerWall( cVector(-9, -8),	cVector(-3, -3), this);
#else //DEBUGRACE is true  Just have one wall.
	cCritterRacerWall *pwall = 
		new cCritterRacerWall( cVector(0, 0),	cVector(5, 5), this);
	//pwall->setThickness(4.0);
	//pwall->setHeight(3.0);
	_seedcount = 0;
#endif //DEBUGRACE
}

void cGameRacer::seedCritters()
{
 	_pbiota->purgeNonPlayerNonWallCritters(); /* Clean out all critters but leave 
		player and	walls. */
	for (int i=0; i < _seedcount; i++)
		new cCritterRacerCar(this);
}

void cGameRacer::initializeView(CPopView *pview)
{
	cGame::initializeView(pview); //You MUST call the base class method here.
	pview->setCursor(((CPopApp*)::AfxGetApp())->_hCursorArrow);
	pview->setUseBackground(CPopView::FULL_BACKGROUND);/* The background type can be
		CPopView::NO_BACKGROUND, CPopView::SIMPLIFIED_BACKGROUND, or 
		CPopView::FULL_BACKGROUND, which often means: nothing, lines, or
		planes&bitmaps, depending on how the skybox is defined. */
	CPoint panepos = pview->paneColRow();
	if (panepos == CPoint(0,0))
	{
		pview->setGraphicsClass(RUNTIME_CLASS(cGraphicsOpenGL));
		pview->pviewpointcritter()->setListener(new cListenerViewerRide());
	}
	if (panepos == CPoint(1,0)) //Use flat map for second view.
	{
		pview->setGraphicsClass(RUNTIME_CLASS(cGraphicsOpenGL)); 
		pview->setUseBackground(CPopView::SIMPLIFIED_BACKGROUND);/* The background type can be
		CPopView::NO_BACKGROUND, CPopView::SIMPLIFIED_BACKGROUND, or 
		CPopView::FULL_BACKGROUND, which often means: nothing, lines, or
		planes&bitmaps, depending on how the skybox is defined. */
	//	pview->pviewpointcritter()->setTrackplayer(TRUE); 
	}
}

void cGameRacer::initializeViewpoint(cCritterViewer *pviewer)
{
	cGame::initializeViewpoint(pviewer); /* No overload.  Note that you don't actually
		care about this call if you plan to ride the player, as then the viewpoint is
		set by the player. */
	pviewer->zoom(0.5);
//#define CHANGERACERVIEWPOINT
#ifdef CHANGERACERVIEWPOINT
	if (pviewer->plistener()->GetRuntimeClass()
		== (RUNTIME_CLASS(cListenerViewerRide)))
	{
		pviewer->setViewpoint(cVector3(0.0, 0.3, -1.0), _border.center());
			//Always make some setViewpoint call simply to put in a default zoom.
		pviewer->zoom(0.5); //Wideangle
		cListenerViewerRide *prider = (cListenerViewerRide*)(pviewer->plistener());
		prider->setOffset((-10.0* cVector::XAXIS + 7.0 * cVector::ZAXIS)); /* This offset is in the coordinate
			system of the player, where the negative X axis is the negative of the
			player's tangent direction, which means stand right behind the player. */
	}
	else //Not riding the player.
	{
		pviewer->zoom(1.5);
		pviewer->setViewpoint(cVector::ZAXIS, pplayer()->position());
	}
#endif //CHANGERACERVIEWPOINT
}

void cGameRacer::fixSkyBox()
{
	cRealBox3 skeleton = _border;
	skeleton.setZRange(-0.1, _border.hiz());
	setSkyBox(&skeleton); 
//	pskybox()->setSideTexture(HIZ, IDB_PALEBLUESCROLLS);//scrolls on ceiling.
	pskybox()->setSideTexture(HIZ, IDB_BIGSKY);//sky on ceiling.
	pskybox()->setSideTexture(LOX, IDB_STREET);//street on wall
	pskybox()->setSideTexture(LOY, IDB_STREET);//street on wall.
	pskybox()->setSideTexture(HIX, IDB_STREET);//street on wall
	pskybox()->setSideTexture(HIY, IDB_STREET);//street on wall.
	/* I do some flips to align the mountain tops and bottoms.
	I probably have more flips than I need. */
	pskybox()->pside(LOY)->rotate(cSpin(PI, cVector::ZAXIS));
	pskybox()->pside(LOY)->rotate(cSpin(PI, cVector::XAXIS));
	pskybox()->pside(HIY)->rotate(cSpin(PI, cVector::ZAXIS));
	pskybox()->pside(HIY)->rotate(cSpin(PI, cVector::XAXIS));
	pskybox()->pside(HIX)->rotate(cSpin(PI, cVector::ZAXIS));
	pskybox()->pside(HIX)->rotate(cSpin(PI, cVector::XAXIS));
	//Or I could make the walls four shades of light green.
//	pskybox()->setSideSolidColor(LOX, RGB(180, 255, 200));
//	pskybox()->setSideSolidColor(HIX, RGB(200, 255, 180));
//	pskybox()->setSideSolidColor(LOY, RGB(180, 255, 180));
//	pskybox()->setSideSolidColor(HIY, RGB(200, 255, 200));
	//Make floor light brown
	pskybox()->setSideSolidColor(LOZ, RGB(200, 100, 100));
}

void cGameRacer::drawBackground(cGraphics *pgraphics, int drawflags)
{
	cGame::drawBackground(pgraphics, drawflags);
}

//=====================cCritterRacerCar===========

cCritterRacerCar::cCritterRacerCar(cGame *pownergame):
cCritterArmedRobot(pownergame)
{
	if (pgame())
		setSprite(pgame()->randomSprite(pgame()->spritetype()));
	cSpriteQuake *pqs = (cSpriteQuake*)(psprite());
	if (pqs)
		pqs->setCorrectionPercents(cVector(0.0, 0.0, 0.1));
	randomize(cCritter::MF_POSITION | cCritter::MF_VELOCITY); 
	setTangent(tangent().getXYUnitVector());
	addForce(new cForceEvadeBullet());
	setArmed(FALSE);
#ifdef CARGRAVITY
	addForce(new cForceGravity(25.0, -cVector::ZAXIS)); //Uses default gravity strength 25.
	addForce(new cForceDrag()); //Uses default friction strength 0.5.
#endif //CARGRAVITY
	setRadius(0.4);
	setDensity(2.0); 
	setMaxspeed(30.0);
}

void cCritterRacerCar::update(CPopView *pactiveview, Real dt)
{
	cCritterArmed::update(pactiveview, dt);
}

void cCritterRacerCar::animate(Real dt)
{
	cCritterArmed::animate(dt);
		/* The base animate method calls 
		copyMotionMatrixToAttitudeMatrix() provided
		the default setAttitudeToMotionLock(TRUE) hasn't
		been overridden in the constructor by calling it
		with the FALSE argument.
			I want my critters to be oriented "heads up",
		so I want their attitude's third column to be the 
		Z axis.  So I force this by making an attitude out
		of these three vectors: (a) for the first column, 
		find a unity XY plane projection of the tangent()
		(c) using the Z axis for the third column,
		and (b) letting the second column be a unity XY
		plane direction perpendicular to (a) and (c) which
		also manages to be roughly in same direction as the normal(). 
			Don't try this trick with the player if you're riding it,
		as it seems to screw thigns up. */
	cVector columna = tangent().getXYUnitVector();
	cVector normalxy = normal().getXYUnitVector();
	cVector columnb = cVector::ZAXIS * columna;
	if (columnb % normalxy < 0) //columnb points wrong way.
		columnb *= -1;
	setAttitude(cMatrix(
		columna, columnb, cVector::ZAXIS, position()));
}

//===================cCritterRacerPlayer================
cCritterRacerPlayer::cCritterRacerPlayer(cGame *pownergame):
cCritterArmedPlayer(pownergame)
{
	setListener(new cListenerCar());
 	setBulletClass(RUNTIME_CLASS(cCritterRacerPlayerBullet));
	setWaitShoot(2.0*cCritterArmed::WAITSHOOT); //Don't shoot as fast as usual.
	setMaxspeed(cGameRacer::MAXPLAYERSPEED);
	setListenerAcceleration(160.0); //So Hopper can overcome gravity
	setRadius(cGameRacer::PLAYERRADIUS);
	setPrismDz(0.4);
	setDensity(2.0); 
#ifdef CARGRAVITY
	addForce(new cForceGravity(5.0, -cVector::ZAXIS)); //Uses default gravity strength 25.
#endif //CARGRAVITY
	addForce(new cForceDrag(1.5)); /* Point of using friction is so he'll
		slow down.  If you make him denser to knock other critters around
		more you may need to turn up friction, it can be higher than 1.0,
		though if you make it way high critter acts weird, with
		"friction" actually reversing its direction. */
#ifndef WEAKBOUNCE //No bounce at all off walls or other critters
	setAbsorberflag(TRUE); //Keeps player from being buffeted about.
#else //WEAKBOUNCE
	setAbsorberflag(FALSE); 
	setBounciness(cGameRacer::RACERPLAYERBOUNCINESS); //Try like 0.2
#endif //WEAKBOUNCE
}

void cCritterRacerPlayer::update(CPopView *pactiveview, Real dt)
{
	cCritterArmedPlayer::update(pactiveview, dt);
}

void cCritterRacerPlayer::animate(Real dt)
{
	cCritterArmedPlayer::animate(dt);
}

//===================cCritterRacerWall
cCritterRacerWall::cCritterRacerWall(const cVector &enda, const cVector &endb,
	 Real nthickness, Real nheight, cGame *pownergame):
cCritterWall(enda, endb, nthickness, nheight, pownergame)
{
	psprite()->setFillColor(cRandomizer::pinstance()->randomColor());
	_movebox.setZRange(-100.0, 100.0); /* So you can comfortably position the 
		3D box so its sprite appears in a nice place, even though your game is
		really using a 2D world. Might be better just to use a 3D _border
		in the first place. */
	moveTo(cVector(_position.x(), _position.y(), height()/2.0));
		/*Let's try and align all the walls with base on the xy plane.*/
}

cCritterRacerWall::cCritterRacerWall(const cVector &enda, const cVector &endb,
	cGame *pownergame):
cCritterWall(enda, endb, cGameRacer::WALLTHICKNESS, cGameRacer::WALLPRISMDZ, pownergame)
{
	setHeight(cRandomizer::pinstance()->randomReal(1.0, 3.0) *
		cGameRacer::WALLPRISMDZ);
	int resourceid = pownergame->randomWallpaperID();
	cSpriteTextureBox *pspritebox =
		new cSpriteTextureBox(pskeleton(), resourceid, 3); //Sets all sides
		/* We'll tile our sprites three times along the long sides, and on the
		short ends, we'll only tile them once, so we reset these two. */
	pspritebox->setSideTexture(LOX, resourceid, 1);
	pspritebox->setSideTexture(HIX, resourceid, 1);
	pspritebox->setSideColors(cRandomizer::pinstance()->randomColor());
		//Use the color in case there's no bitmap.
	setSprite(pspritebox);
//	setSprite(new cSpriteTextureBox(pskeleton(), IDB_SUN));
	_movebox.setZRange(-100.0, 100.0); /* So you can comfortably position the 
		3D box so its sprite appears in a nice place, even though your game is
		really using a 2D world. Might be better just to use a 3D _border
		in the first place. */
	_dragbox.setZRange(-100.0, 100.0); /* Also need to fix this so that when
		we drag, it doesn't get clamped down. */
	moveTo(cVector(_position.x(), _position.y(), -0.1 + height()/2.0));
		/* Align its base to be just a hair under the xy plane. I put
		it a hair under just to be sure I don't get screwy edge collisions
		when things in the plane bump it.*/
}

//====================cCritterRacerPlayerBullet==================
cCritterRacerPlayerBullet::cCritterRacerPlayerBullet()
{
	setRadius(2.0*cCritter::BULLETRADIUS); //Make bigger than usual.
}

void cCritterRacerPlayerBullet::initialize(cCritterArmed *pshooter)
{
	cCritterBullet::initialize(pshooter);
	clearForcearray();/* The cCritterBullet::initialize copies the physics 
		forces from pshooter to the bullet, and if I don't want the car's
		friction, I have to turn it off here by clearing the force array. */
}

cCritterRacerPlayerBullet::isTarget(cCritter *pcritter)
{
	return cCritterBullet::isTarget(pcritter) && 
		pcritter->GetRuntimeClass() != RUNTIME_CLASS(cCritterRacerWall);
}

//============Enemy bullets==============
cCritterCarBullet::cCritterCarBullet()
{
	setSprite(new cSpriteSphere());
	setRadius(2.0*cCritter::BULLETRADIUS); //Make bigger than usual.
}

void cCritterCarBullet::initialize(cCritterArmed *pshooter)
{
	cCritterBullet::initialize(pshooter);
	clearForcearray();/* The cCritterBullet::initialize copies the physics 
		forces from pshooter to the bullet, and if I don't want the car's
		friction, I have to turn it off here by clearing the force array. */
}

cCritterCarBullet::isTarget(cCritter *pcritter)
{
	return pcritter == pplayer();
}

